import nock from 'nock';
import axios from 'axios';
import httpAdapter from 'axios/lib/adapters/http';

import fs from 'fs';
import path from 'path';
import rimraf from 'rimraf';

import loader from '../src';

axios.defaults.adapter = httpAdapter;

const dir = fs.mkdtempSync('/var/tmp/');

afterAll(() => {
  rimraf(dir, () => {});
});

test('save page', () => {
  expect.assertions(3);

  const host = 'http://test.com';

  nock(host)
    .get('/page')
    .reply(200, fs.readFileSync(path.resolve('__tests__/fixtures/page.html'), 'utf8'));

  const favicon = fs.readFileSync(path.resolve('__tests__/fixtures/favicon.ico'));
  nock(host)
    .get('/assets/icons/default/favicon.ico')
    .reply(200, favicon);

  const ruby = fs.readFileSync(path.resolve('__tests__/fixtures/ruby.png'));
  nock(host)
    .get('/images/ruby.png')
    .reply(200, ruby);

  const page = `${host}/page`;
  const mainFile = path.join(dir, 'test-com-page.html');
  const dirFiles = path.join(dir, 'test-com-page_files');
  const faviconFile = path.join(dirFiles, 'assets-icons-default-favicon.ico');
  const rubyFile = path.join(dirFiles, 'images-ruby.png');
  return loader(page, dir)
    .then(() => fs.readFileSync(mainFile, 'utf8'))
    .then(data => expect(data).toBe(fs.readFileSync(path.resolve('__tests__/fixtures/savedPage.html'), 'utf8')))
    .then(() => fs.readFileSync(faviconFile, 'utf8'))
    .then(data => expect(data).toBe(favicon.toString()))
    .then(() => fs.readFileSync(rubyFile, 'utf8'))
    .then(data => expect(data).toBe(ruby.toString()));
});
