import axios from 'axios';
import fs from 'mz/fs';
import path from 'path';
import url from 'url';

import string from './string';
import parse from './parse';

const isLocalLink = (link, hostname) => !url.parse(link).hostname
  || url.parse(link).hostname === hostname;

const newLink = (link, host, dir) => {
  if (isLocalLink(link, host)) {
    return path.join(dir, string.urlToKebabCase(link));
  }
  return link;
};

const copyFile = (link, urlData, filepath) => {
  const fullLink = !url.parse(link).hostname
    ? url.format({ ...urlData, pathname: link }) : link;

  return axios({
    method: 'get',
    url: fullLink,
    responseType: 'stream' })
    .then((res) => {
      if (res.status !== 200) {
        throw (new Error(res.statusText));
      }
      return res.data.pipe(fs.createWriteStream(filepath));
    });
};

export default (page, dir) => {
  const { protocol, hostname } = url.parse(page);
  const urlData = { protocol, hostname };
  const pageName = string.urlToKebabCase(page);
  const filename = `${pageName}.html`;
  const simpleDirFiles = `${pageName}_files`;

  return axios.get(page)
    .then((res) => {
      if (res.status !== 200) {
        throw (new Error(res.statusText));
      }
      return res.data;
    })
    .then((html) => {
      const links = parse(html);
      const replacements = links.map((link) => {
        const rep = { newLink: newLink(link, urlData.hostname, simpleDirFiles), link };
        return rep;
      }).filter(rep => rep.newLink !== rep.link);
      const newHtml = replacements.reduce((acc, rep) =>
        acc.replace(new RegExp(rep.link, 'g'), rep.newLink), html);
      if (links.length === 0) {
        return newHtml;
      }
      const copyFiles = replacements.map(rep =>
        copyFile(rep.link, urlData, path.join(dir, rep.newLink)));
      return fs.mkdir(path.join(dir, simpleDirFiles))
        .then(() => Promise.all(copyFiles))
        .then(() => newHtml);
    })
    .then(html => fs.writeFile(path.join(dir, filename), html));
};
